@@ -1336,10 +1336,17 @@ static int s_build_canonical_stable_header_list(
             }
         }
 
-        /*
-         * x-amz-content-sha256 (optional)
-         */
-        if (state->config.signed_body_header == AWS_SBHT_X_AMZ_CONTENT_SHA256) {
+        /* NOTE: Update MAX_AUTHORIZATION_HEADER_COUNT if more headers added */
+    }
+
+    /*
+     * x-amz-content-sha256 (optional)
+     */
+    if (state->config.signed_body_header == AWS_SBHT_X_AMZ_CONTENT_SHA256) {
+        if (state->config.signature_type == AWS_ST_HTTP_REQUEST_HEADERS ||
+            (state->config.signature_type == AWS_ST_HTTP_REQUEST_QUERY_PARAMS &&
+             aws_byte_cursor_eq(&state->config.signed_body_value, &g_aws_signed_body_value_unsigned_payload))) {
+            /* Add the x-amz-content-sha256 header for UNSIGNED-PAYLOAD when signing via query params as well. */
             if (s_add_authorization_header(
                     state,
                     stable_header_list,
@@ -1349,8 +1356,6 @@ static int s_build_canonical_stable_header_list(
                 return AWS_OP_ERR;
             }
         }
-
-        /* NOTE: Update MAX_AUTHORIZATION_HEADER_COUNT if more headers added */
     }
 
     *out_required_capacity += aws_array_list_length(stable_header_list) * 2; /*  ':' + '\n' per header */
@@ -2088,7 +2093,7 @@ int aws_signing_build_canonical_request(struct aws_signing_state_aws *state) {
             return s_apply_existing_canonical_request(state);
 
         default:
-            return AWS_OP_ERR;
+            return aws_raise_error(AWS_AUTH_SIGNING_UNSUPPORTED_SIGNATURE_TYPE);
     }
 }
 
